﻿
//
// Version 3.0; 2010-07-31
// Using Fourmilab Calendar Converter by John Walker
//
// * This file uses the faster but unofficial approximating functions (2820 algorithm)
// * Do not use unless absolutely necessary. Good between 1342 and 1403.
//
//
// Constructors:
//  var d = new IranianDate();
//  var d = new IranianDate(1362, 3, 3);
//  var d = new IranianDate(new Date(2001, 8, 11));
//  var d = new IranianDate(new IranianDate(1357, 10, 22));
//  var d = new IranianDate("جمعه 22 خرداد 1388");
//  var d = new IranianDate("29 اسفند 1329");
//  var d = new IranianDate("1332-4-28");                   // 4=مرداد
//
// Methods:
//  setIranianDate(y, m, d)
//  getFullYear(), setFullYear(y)
//  getYear(), setYear(y)           (deprecated, 88 instead of 1388)
//  getMonth(), setMonth(m)         (0..11, 0=Farvardin, 1=Ordibehesht, ..., 11=Esfand)
//  getDate(), setDate(d)           (1..31)
//  getDay()                        (0..6, 0=Saturday, 1=Sunday, ..., 6=Friday, differs from JavaScript Date)
//
//  toDateString()                  (returns "چهارشنبه 28 مرداد 1332")
//  toDate(), fromDate(d)           (convert to/from Gregorian, returns/gets JavaScript Date)
//  getMonthLength()                (how many days in this month, takes care of leap year)
//  _MonthFromName(name)            (internal, converts name to number, eg. _MonthFromName("اردیبهشت") returns 1)
//
// Properties
//  MonthNames = Array("فروردین", "اردیبهشت", "خرداد", "تیر", "مرداد", "شهریور", "مهر", "آبان", "آذر", "دی", "بهمن", "اسفند");
//  DayOfWeekNames = Array("شنبه", "یک‌شنبه", "دوشنبه", "سه‌شنبه", "چهارشنبه", "پنج‌شنبه", "جمعه");
//


function IranianDate(y, m, d) {
    if ((typeof m == "number") || (typeof d == "number")) {
        this.setIranianDate(y, m, d);
    }
    else if (y instanceof Date) {
        this.fromDate(y);
    }
    else if (y instanceof IranianDate) {
        this.setIranianDate(y.FullYear, y.Month, y.Date);
    }
    else if ((typeof y == "string") && (parts = y.split(" ")).length == 4) {
        this.setIranianDate(parts[3], this._MonthFromName(parts[2]), parts[1]);
    }
    else if ((typeof y == "string") && parts.length == 3) {
        this.setIranianDate(parts[2], this._MonthFromName(parts[1]), parts[0]);
    }
    else if ((typeof y == "string") && (parts = y.split("-")).length == 3) {
        this.setIranianDate(parts[0], parts[1], parts[2]);
    }
    else {
        this.fromDate(new Date());
    }
}


IranianDate.prototype.getFullYear = function () { return this.FullYear; }
IranianDate.prototype.getYear = function () { return this.getFullYear() - 1300; }
IranianDate.prototype.getMonth = function () { return this.Month; }
IranianDate.prototype.getDate = function () { return this.Date; }
IranianDate.prototype.getDay = function () { return (this.toDate().getDay() + 1) % 7; }
IranianDate.prototype.toDateString = function () { return this.DayOfWeekNames[this.getDay()] + " " + this.Date + " " + this.MonthNames[this.Month] + " " + this.FullYear; }
IranianDate.prototype.setFullYear = function (y) { this.setIranianDate(y, this.Month, this.Date); }
IranianDate.prototype.setYear = function (y) { this.setFullYear(y + 1300); }
IranianDate.prototype.setMonth = function (m) { this.setIranianDate(this.FullYear, m, this.Date); }
IranianDate.prototype.setDate = function (d) { this.setIranianDate(this.FullYear, this.Month, d); }

IranianDate.prototype.getMonthLength = function () {
    var lastday = 31;
    if (this.Month > 5) lastday = 30;
    if (this.Month == 11 && !leap_persiana(this.FullYear)) lastday = 29;
    return lastday;
}

IranianDate.prototype.MonthNames = Array("فروردین", "اردیبهشت", "خرداد", "تیر", "مرداد", "شهریور", "مهر", "آبان", "آذر", "دی", "بهمن", "اسفند");
IranianDate.prototype.DayOfWeekNames = Array("شنبه", "یک‌شنبه", "دوشنبه", "سه‌شنبه", "چهارشنبه", "پنج‌شنبه", "جمعه");
IranianDate.prototype._MonthFromName = function (name) {
    for (var i = 0; i < 12; i++)
        if (this.MonthNames[i] == name) return i;
    return -1;
}


// These functions use Fourmilab Calendar Converter by John Walker:

IranianDate.prototype.setIranianDate = function (y, m, d) {
    var p = jd_to_persian(persian_to_jd(parseInt(y), parseInt(m) + 1, parseInt(d)));
    this.FullYear = p[0];
    this.Month = p[1] - 1;
    this.Date = p[2];
}

IranianDate.prototype.fromDate = function (d) {
    var p = jd_to_persian(gregorian_to_jd(d.getFullYear(), d.getMonth() + 1, d.getDate()));
    this.FullYear = p[0];
    this.Month = p[1] - 1;
    this.Date = p[2];
}

IranianDate.prototype.toDate = function () {
    var p = jd_to_gregorian(persian_to_jd(this.FullYear, this.Month + 1, this.Date));
    var d = new Date(p[0], p[1] - 1, p[2]);
    d.setDate(p[2]);
    return d;
}

